/* This file was generated by syncqt. */
#ifndef QT_QT3DQUICKSCENE2D_VERSION_H
#define QT_QT3DQUICKSCENE2D_VERSION_H

#define QT3DQUICKSCENE2D_VERSION_STR "5.15.2"

#define QT3DQUICKSCENE2D_VERSION 0x050F02

#endif // QT_QT3DQUICKSCENE2D_VERSION_H
