/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the QtGui module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef UIAPROPERTYIDS_H
#define UIAPROPERTYIDS_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API. It exists purely as an
// implementation detail. This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#define UIA_RuntimeIdPropertyId                                30000
#define UIA_BoundingRectanglePropertyId                        30001
#define UIA_ProcessIdPropertyId                                30002
#define UIA_ControlTypePropertyId                              30003
#define UIA_LocalizedControlTypePropertyId                     30004
#define UIA_NamePropertyId                                     30005
#define UIA_AcceleratorKeyPropertyId                           30006
#define UIA_AccessKeyPropertyId                                30007
#define UIA_HasKeyboardFocusPropertyId                         30008
#define UIA_IsKeyboardFocusablePropertyId                      30009
#define UIA_IsEnabledPropertyId                                30010
#define UIA_AutomationIdPropertyId                             30011
#define UIA_ClassNamePropertyId                                30012
#define UIA_HelpTextPropertyId                                 30013
#define UIA_ClickablePointPropertyId                           30014
#define UIA_CulturePropertyId                                  30015
#define UIA_IsControlElementPropertyId                         30016
#define UIA_IsContentElementPropertyId                         30017
#define UIA_LabeledByPropertyId                                30018
#define UIA_IsPasswordPropertyId                               30019
#define UIA_NativeWindowHandlePropertyId                       30020
#define UIA_ItemTypePropertyId                                 30021
#define UIA_IsOffscreenPropertyId                              30022
#define UIA_OrientationPropertyId                              30023
#define UIA_FrameworkIdPropertyId                              30024
#define UIA_IsRequiredForFormPropertyId                        30025
#define UIA_ItemStatusPropertyId                               30026
#define UIA_IsDockPatternAvailablePropertyId                   30027
#define UIA_IsExpandCollapsePatternAvailablePropertyId         30028
#define UIA_IsGridItemPatternAvailablePropertyId               30029
#define UIA_IsGridPatternAvailablePropertyId                   30030
#define UIA_IsInvokePatternAvailablePropertyId                 30031
#define UIA_IsMultipleViewPatternAvailablePropertyId           30032
#define UIA_IsRangeValuePatternAvailablePropertyId             30033
#define UIA_IsScrollPatternAvailablePropertyId                 30034
#define UIA_IsScrollItemPatternAvailablePropertyId             30035
#define UIA_IsSelectionItemPatternAvailablePropertyId          30036
#define UIA_IsSelectionPatternAvailablePropertyId              30037
#define UIA_IsTablePatternAvailablePropertyId                  30038
#define UIA_IsTableItemPatternAvailablePropertyId              30039
#define UIA_IsTextPatternAvailablePropertyId                   30040
#define UIA_IsTogglePatternAvailablePropertyId                 30041
#define UIA_IsTransformPatternAvailablePropertyId              30042
#define UIA_IsValuePatternAvailablePropertyId                  30043
#define UIA_IsWindowPatternAvailablePropertyId                 30044
#define UIA_ValueValuePropertyId                               30045
#define UIA_ValueIsReadOnlyPropertyId                          30046
#define UIA_RangeValueValuePropertyId                          30047
#define UIA_RangeValueIsReadOnlyPropertyId                     30048
#define UIA_RangeValueMinimumPropertyId                        30049
#define UIA_RangeValueMaximumPropertyId                        30050
#define UIA_RangeValueLargeChangePropertyId                    30051
#define UIA_RangeValueSmallChangePropertyId                    30052
#define UIA_ScrollHorizontalScrollPercentPropertyId            30053
#define UIA_ScrollHorizontalViewSizePropertyId                 30054
#define UIA_ScrollVerticalScrollPercentPropertyId              30055
#define UIA_ScrollVerticalViewSizePropertyId                   30056
#define UIA_ScrollHorizontallyScrollablePropertyId             30057
#define UIA_ScrollVerticallyScrollablePropertyId               30058
#define UIA_SelectionSelectionPropertyId                       30059
#define UIA_SelectionCanSelectMultiplePropertyId               30060
#define UIA_SelectionIsSelectionRequiredPropertyId             30061
#define UIA_GridRowCountPropertyId                             30062
#define UIA_GridColumnCountPropertyId                          30063
#define UIA_GridItemRowPropertyId                              30064
#define UIA_GridItemColumnPropertyId                           30065
#define UIA_GridItemRowSpanPropertyId                          30066
#define UIA_GridItemColumnSpanPropertyId                       30067
#define UIA_GridItemContainingGridPropertyId                   30068
#define UIA_DockDockPositionPropertyId                         30069
#define UIA_ExpandCollapseExpandCollapseStatePropertyId        30070
#define UIA_MultipleViewCurrentViewPropertyId                  30071
#define UIA_MultipleViewSupportedViewsPropertyId               30072
#define UIA_WindowCanMaximizePropertyId                        30073
#define UIA_WindowCanMinimizePropertyId                        30074
#define UIA_WindowWindowVisualStatePropertyId                  30075
#define UIA_WindowWindowInteractionStatePropertyId             30076
#define UIA_WindowIsModalPropertyId                            30077
#define UIA_WindowIsTopmostPropertyId                          30078
#define UIA_SelectionItemIsSelectedPropertyId                  30079
#define UIA_SelectionItemSelectionContainerPropertyId          30080
#define UIA_TableRowHeadersPropertyId                          30081
#define UIA_TableColumnHeadersPropertyId                       30082
#define UIA_TableRowOrColumnMajorPropertyId                    30083
#define UIA_TableItemRowHeaderItemsPropertyId                  30084
#define UIA_TableItemColumnHeaderItemsPropertyId               30085
#define UIA_ToggleToggleStatePropertyId                        30086
#define UIA_TransformCanMovePropertyId                         30087
#define UIA_TransformCanResizePropertyId                       30088
#define UIA_TransformCanRotatePropertyId                       30089
#define UIA_IsLegacyIAccessiblePatternAvailablePropertyId      30090
#define UIA_LegacyIAccessibleChildIdPropertyId                 30091
#define UIA_LegacyIAccessibleNamePropertyId                    30092
#define UIA_LegacyIAccessibleValuePropertyId                   30093
#define UIA_LegacyIAccessibleDescriptionPropertyId             30094
#define UIA_LegacyIAccessibleRolePropertyId                    30095
#define UIA_LegacyIAccessibleStatePropertyId                   30096
#define UIA_LegacyIAccessibleHelpPropertyId                    30097
#define UIA_LegacyIAccessibleKeyboardShortcutPropertyId        30098
#define UIA_LegacyIAccessibleSelectionPropertyId               30099
#define UIA_LegacyIAccessibleDefaultActionPropertyId           30100
#define UIA_AriaRolePropertyId                                 30101
#define UIA_AriaPropertiesPropertyId                           30102
#define UIA_IsDataValidForFormPropertyId                       30103
#define UIA_ControllerForPropertyId                            30104
#define UIA_DescribedByPropertyId                              30105
#define UIA_FlowsToPropertyId                                  30106
#define UIA_ProviderDescriptionPropertyId                      30107
#define UIA_IsItemContainerPatternAvailablePropertyId          30108
#define UIA_IsVirtualizedItemPatternAvailablePropertyId        30109
#define UIA_IsSynchronizedInputPatternAvailablePropertyId      30110
#define UIA_OptimizeForVisualContentPropertyId                 30111
#define UIA_IsObjectModelPatternAvailablePropertyId            30112
#define UIA_AnnotationAnnotationTypeIdPropertyId               30113
#define UIA_AnnotationAnnotationTypeNamePropertyId             30114
#define UIA_AnnotationAuthorPropertyId                         30115
#define UIA_AnnotationDateTimePropertyId                       30116
#define UIA_AnnotationTargetPropertyId                         30117
#define UIA_IsAnnotationPatternAvailablePropertyId             30118
#define UIA_IsTextPattern2AvailablePropertyId                  30119
#define UIA_StylesStyleIdPropertyId                            30120
#define UIA_StylesStyleNamePropertyId                          30121
#define UIA_StylesFillColorPropertyId                          30122
#define UIA_StylesFillPatternStylePropertyId                   30123
#define UIA_StylesShapePropertyId                              30124
#define UIA_StylesFillPatternColorPropertyId                   30125
#define UIA_StylesExtendedPropertiesPropertyId                 30126
#define UIA_IsStylesPatternAvailablePropertyId                 30127
#define UIA_IsSpreadsheetPatternAvailablePropertyId            30128
#define UIA_SpreadsheetItemFormulaPropertyId                   30129
#define UIA_SpreadsheetItemAnnotationObjectsPropertyId         30130
#define UIA_SpreadsheetItemAnnotationTypesPropertyId           30131
#define UIA_IsSpreadsheetItemPatternAvailablePropertyId        30132
#define UIA_Transform2CanZoomPropertyId                        30133
#define UIA_IsTransformPattern2AvailablePropertyId             30134
#define UIA_LiveSettingPropertyId                              30135
#define UIA_IsTextChildPatternAvailablePropertyId              30136
#define UIA_IsDragPatternAvailablePropertyId                   30137
#define UIA_DragIsGrabbedPropertyId                            30138
#define UIA_DragDropEffectPropertyId                           30139
#define UIA_DragDropEffectsPropertyId                          30140
#define UIA_IsDropTargetPatternAvailablePropertyId             30141
#define UIA_DropTargetDropTargetEffectPropertyId               30142
#define UIA_DropTargetDropTargetEffectsPropertyId              30143
#define UIA_DragGrabbedItemsPropertyId                         30144
#define UIA_Transform2ZoomLevelPropertyId                      30145
#define UIA_Transform2ZoomMinimumPropertyId                    30146
#define UIA_Transform2ZoomMaximumPropertyId                    30147
#define UIA_FlowsFromPropertyId                                30148
#define UIA_IsTextEditPatternAvailablePropertyId               30149
#define UIA_IsPeripheralPropertyId                             30150
#define UIA_IsCustomNavigationPatternAvailablePropertyId       30151
#define UIA_PositionInSetPropertyId                            30152
#define UIA_SizeOfSetPropertyId                                30153
#define UIA_LevelPropertyId                                    30154
#define UIA_AnnotationTypesPropertyId                          30155
#define UIA_AnnotationObjectsPropertyId                        30156
#define UIA_LandmarkTypePropertyId                             30157
#define UIA_LocalizedLandmarkTypePropertyId                    30158
#define UIA_FullDescriptionPropertyId                          30159
#define UIA_FillColorPropertyId                                30160
#define UIA_OutlineColorPropertyId                             30161
#define UIA_FillTypePropertyId                                 30162
#define UIA_VisualEffectsPropertyId                            30163
#define UIA_OutlineThicknessPropertyId                         30164
#define UIA_CenterPointPropertyId                              30165
#define UIA_RotationPropertyId                                 30166
#define UIA_SizePropertyId                                     30167
#define UIA_IsDialogPropertyId                                 30174

#endif
