/****************************************************************************
**
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the QtGui module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or (at your option) the GNU General
** Public license version 3 or any later version approved by the KDE Free
** Qt Foundation. The licenses are as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-2.0.html and
** https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef UIAATTRIBUTEIDS_H
#define UIAATTRIBUTEIDS_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API. It exists purely as an
// implementation detail. This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#define UIA_AnimationStyleAttributeId            40000
#define UIA_BackgroundColorAttributeId           40001
#define UIA_BulletStyleAttributeId               40002
#define UIA_CapStyleAttributeId                  40003
#define UIA_CultureAttributeId                   40004
#define UIA_FontNameAttributeId                  40005
#define UIA_FontSizeAttributeId                  40006
#define UIA_FontWeightAttributeId                40007
#define UIA_ForegroundColorAttributeId           40008
#define UIA_HorizontalTextAlignmentAttributeId   40009
#define UIA_IndentationFirstLineAttributeId      40010
#define UIA_IndentationLeadingAttributeId        40011
#define UIA_IndentationTrailingAttributeId       40012
#define UIA_IsHiddenAttributeId                  40013
#define UIA_IsItalicAttributeId                  40014
#define UIA_IsReadOnlyAttributeId                40015
#define UIA_IsSubscriptAttributeId               40016
#define UIA_IsSuperscriptAttributeId             40017
#define UIA_MarginBottomAttributeId              40018
#define UIA_MarginLeadingAttributeId             40019
#define UIA_MarginTopAttributeId                 40020
#define UIA_MarginTrailingAttributeId            40021
#define UIA_OutlineStylesAttributeId             40022
#define UIA_OverlineColorAttributeId             40023
#define UIA_OverlineStyleAttributeId             40024
#define UIA_StrikethroughColorAttributeId        40025
#define UIA_StrikethroughStyleAttributeId        40026
#define UIA_TabsAttributeId                      40027
#define UIA_TextFlowDirectionsAttributeId        40028
#define UIA_UnderlineColorAttributeId            40029
#define UIA_UnderlineStyleAttributeId            40030
#define UIA_AnnotationTypesAttributeId           40031
#define UIA_AnnotationObjectsAttributeId         40032
#define UIA_StyleNameAttributeId                 40033
#define UIA_StyleIdAttributeId                   40034
#define UIA_LinkAttributeId                      40035
#define UIA_IsActiveAttributeId                  40036
#define UIA_SelectionActiveEndAttributeId        40037
#define UIA_CaretPositionAttributeId             40038
#define UIA_CaretBidiModeAttributeId             40039
#define UIA_LineSpacingAttributeId               40040
#define UIA_BeforeParagraphSpacingAttributeId    40041
#define UIA_AfterParagraphSpacingAttributeId     40042
#define UIA_SayAsInterpretAsAttributeId          40043

#endif
