#define QT_FEATURE_cxx14_make_unique 1
#define QT_FEATURE_qml_animation 1
#define QT_FEATURE_qml_devtools 1
#define QT_FEATURE_qml_itemmodel 1
#define QT_FEATURE_qml_jit 1
#define QT_FEATURE_qml_locale 1
#define QT_FEATURE_qml_preview 1
#define QT_FEATURE_qml_profiler 1
#define QT_FEATURE_qml_python 1
#define QT_FEATURE_qml_sequence_object 1
#define QT_FEATURE_qml_worker_script 1
#define QT_FEATURE_qml_xml_http_request 1
